pub struct Range<T>where
T: RangeSubType,{ /* private fields */ }Expand description
A safe deconstruction of a Postgres pg_sys::RangeType struct.
In spirit, Postgres ranges are not dissimilar from Rust ranges, however they are represented
quite differently. You’ll use a RangeBound for the lower and upper bounds of a Postgres
Range.
Unlike Rust, Postgres also has the concept of an “empty” range. Such ranges are constructed via
the Range::empty() function. As such, expect the various direct accessor methods on Range
to return Option<&RangeBound<T>> or Option<(RangeBound<T>, RangeBound<T>)>.
pgx provides From implementations for Rust’s built-in range types for easy conversion into
a Postgres range. For example:
use pgx::Range;
let r: Range<i32> = (1..10).into();Implementations
Sourceimpl<T> Range<T>where
T: RangeSubType,
impl<T> Range<T>where
T: RangeSubType,
Sourcepub fn new<L, U>(lower: L, upper: U) -> Range<T>
pub fn new<L, U>(lower: L, upper: U) -> Range<T>
Builds a new Range with bounds.
Examples
use pgx::{Range, RangeBound};
let _ = Range::<i32>::new(1, 10); // `(1..=10)`
let _ = Range::<i32>::new(None, 10); // `(..=10)`
let _ = Range::<i32>::new(1, None); // `(1..)`
let _ = Range::<i32>::new(None, RangeBound::Exclusive(10)); // `(..10)`
let _ = Range::<i32>::new(1, RangeBound::Exclusive(10)); // (`1..10)`
let _ = Range::<i32>::new(None, None); // `(..)`
let _ = Range::<i32>::new(RangeBound::Infinite, RangeBound::Infinite); // `(..)`Sourcepub fn empty() -> Range<T>
pub fn empty() -> Range<T>
Builds an “empty” range
Unlike Rust ranges (from std::ops::), Postgres ranges can be empty, meaning they don’t
represent any range of values.
Sourcepub fn infinite() -> Range<T>
pub fn infinite() -> Range<T>
Builds an “infinite” range. This is equivalent to Rust’s std::ops::RangeFull ((..)).
Sourcepub fn lower(&self) -> Option<&RangeBound<T>>
pub fn lower(&self) -> Option<&RangeBound<T>>
Returns the lower RangeBound
Sourcepub fn upper(&self) -> Option<&RangeBound<T>>
pub fn upper(&self) -> Option<&RangeBound<T>>
Returns the upper RangeBound
Sourcepub fn is_infinite(&self) -> bool
pub fn is_infinite(&self) -> bool
Returns true if the range is “infinite”. This is equivalent to Rust’s std::ops::RangeFull ((..))
Sourcepub fn into_inner(self) -> Option<(RangeBound<T>, RangeBound<T>)>
pub fn into_inner(self) -> Option<(RangeBound<T>, RangeBound<T>)>
Consumes self and returns the internal representation, which can be easily mapped or
unwrapped.
A return value of Option::None indicates that this range represents the “empty” range.
Sourcepub fn take(&mut self) -> Option<(RangeBound<T>, RangeBound<T>)>
pub fn take(&mut self) -> Option<(RangeBound<T>, RangeBound<T>)>
Takes the bounds out of this Range and converts self to represent the “empty” range.
A return value of Option::None indicates that this range already represents the “empty” range.
Sourcepub fn replace(
&mut self,
new: Option<(RangeBound<T>, RangeBound<T>)>,
) -> Option<(RangeBound<T>, RangeBound<T>)>
pub fn replace( &mut self, new: Option<(RangeBound<T>, RangeBound<T>)>, ) -> Option<(RangeBound<T>, RangeBound<T>)>
Replace the bounds of this Range, returning the old bounds.
An Option::None will replace this with the “empty” range.
Methods from Deref<Target = Option<(RangeBound<T>, RangeBound<T>)>>
1.0.0 · Sourcepub fn as_ref(&self) -> Option<&T>
pub fn as_ref(&self) -> Option<&T>
Converts from &Option<T> to Option<&T>.
Examples
Calculates the length of an Option<String> as an Option<usize>
without moving the String. The map method takes the self argument by value,
consuming the original, so this technique uses as_ref to first take an Option to a
reference to the value inside the original.
let text: Option<String> = Some("Hello, world!".to_string());
// First, cast `Option<String>` to `Option<&String>` with `as_ref`,
// then consume *that* with `map`, leaving `text` on the stack.
let text_length: Option<usize> = text.as_ref().map(|s| s.len());
println!("still can print text: {text:?}");1.0.0 · Sourcepub fn as_mut(&mut self) -> Option<&mut T>
pub fn as_mut(&mut self) -> Option<&mut T>
Converts from &mut Option<T> to Option<&mut T>.
Examples
let mut x = Some(2);
match x.as_mut() {
Some(v) => *v = 42,
None => {},
}
assert_eq!(x, Some(42));1.75.0 · Sourcepub fn as_slice(&self) -> &[T]
pub fn as_slice(&self) -> &[T]
Returns a slice of the contained value, if any. If this is None, an
empty slice is returned. This can be useful to have a single type of
iterator over an Option or slice.
Note: Should you have an Option<&T> and wish to get a slice of T,
you can unpack it via opt.map_or(&[], std::slice::from_ref).
Examples
assert_eq!(
[Some(1234).as_slice(), None.as_slice()],
[&[1234][..], &[][..]],
);The inverse of this function is (discounting
borrowing) [_]::first:
for i in [Some(1234_u16), None] {
assert_eq!(i.as_ref(), i.as_slice().first());
}1.75.0 · Sourcepub fn as_mut_slice(&mut self) -> &mut [T]
pub fn as_mut_slice(&mut self) -> &mut [T]
Returns a mutable slice of the contained value, if any. If this is
None, an empty slice is returned. This can be useful to have a
single type of iterator over an Option or slice.
Note: Should you have an Option<&mut T> instead of a
&mut Option<T>, which this method takes, you can obtain a mutable
slice via opt.map_or(&mut [], std::slice::from_mut).
Examples
assert_eq!(
[Some(1234).as_mut_slice(), None.as_mut_slice()],
[&mut [1234][..], &mut [][..]],
);The result is a mutable slice of zero or one items that points into
our original Option:
let mut x = Some(1234);
x.as_mut_slice()[0] += 1;
assert_eq!(x, Some(1235));The inverse of this method (discounting borrowing)
is [_]::first_mut:
assert_eq!(Some(123).as_mut_slice().first_mut(), Some(&mut 123))1.40.0 · Sourcepub fn as_deref(&self) -> Option<&<T as Deref>::Target>where
T: Deref,
pub fn as_deref(&self) -> Option<&<T as Deref>::Target>where
T: Deref,
Converts from Option<T> (or &Option<T>) to Option<&T::Target>.
Leaves the original Option in-place, creating a new one with a reference
to the original one, additionally coercing the contents via Deref.
Examples
let x: Option<String> = Some("hey".to_owned());
assert_eq!(x.as_deref(), Some("hey"));
let x: Option<String> = None;
assert_eq!(x.as_deref(), None);1.40.0 · Sourcepub fn as_deref_mut(&mut self) -> Option<&mut <T as Deref>::Target>where
T: DerefMut,
pub fn as_deref_mut(&mut self) -> Option<&mut <T as Deref>::Target>where
T: DerefMut,
Converts from Option<T> (or &mut Option<T>) to Option<&mut T::Target>.
Leaves the original Option in-place, creating a new one containing a mutable reference to
the inner type’s Deref::Target type.
Examples
let mut x: Option<String> = Some("hey".to_owned());
assert_eq!(x.as_deref_mut().map(|x| {
x.make_ascii_uppercase();
x
}), Some("HEY".to_owned().as_mut_str()));1.0.0 · Sourcepub fn iter(&self) -> Iter<'_, T>
pub fn iter(&self) -> Iter<'_, T>
Returns an iterator over the possibly contained value.
Examples
let x = Some(4);
assert_eq!(x.iter().next(), Some(&4));
let x: Option<u32> = None;
assert_eq!(x.iter().next(), None);1.0.0 · Sourcepub fn iter_mut(&mut self) -> IterMut<'_, T>
pub fn iter_mut(&mut self) -> IterMut<'_, T>
Returns a mutable iterator over the possibly contained value.
Examples
let mut x = Some(4);
match x.iter_mut().next() {
Some(v) => *v = 42,
None => {},
}
assert_eq!(x, Some(42));
let mut x: Option<u32> = None;
assert_eq!(x.iter_mut().next(), None);1.53.0 · Sourcepub fn insert(&mut self, value: T) -> &mut T
pub fn insert(&mut self, value: T) -> &mut T
Inserts value into the option, then returns a mutable reference to it.
If the option already contains a value, the old value is dropped.
See also Option::get_or_insert, which doesn’t update the value if
the option already contains Some.
Example
let mut opt = None;
let val = opt.insert(1);
assert_eq!(*val, 1);
assert_eq!(opt.unwrap(), 1);
let val = opt.insert(2);
assert_eq!(*val, 2);
*val = 3;
assert_eq!(opt.unwrap(), 3);1.20.0 · Sourcepub fn get_or_insert(&mut self, value: T) -> &mut T
pub fn get_or_insert(&mut self, value: T) -> &mut T
Inserts value into the option if it is None, then
returns a mutable reference to the contained value.
See also Option::insert, which updates the value even if
the option already contains Some.
Examples
let mut x = None;
{
let y: &mut u32 = x.get_or_insert(5);
assert_eq!(y, &5);
*y = 7;
}
assert_eq!(x, Some(7));1.83.0 · Sourcepub fn get_or_insert_default(&mut self) -> &mut Twhere
T: Default,
pub fn get_or_insert_default(&mut self) -> &mut Twhere
T: Default,
1.20.0 · Sourcepub fn get_or_insert_with<F>(&mut self, f: F) -> &mut Twhere
F: FnOnce() -> T,
pub fn get_or_insert_with<F>(&mut self, f: F) -> &mut Twhere
F: FnOnce() -> T,
1.80.0 · Sourcepub fn take_if<P>(&mut self, predicate: P) -> Option<T>
pub fn take_if<P>(&mut self, predicate: P) -> Option<T>
Takes the value out of the option, but only if the predicate evaluates to
true on a mutable reference to the value.
In other words, replaces self with None if the predicate returns true.
This method operates similar to Option::take but conditional.
Examples
let mut x = Some(42);
let prev = x.take_if(|v| if *v == 42 {
*v += 1;
false
} else {
false
});
assert_eq!(x, Some(43));
assert_eq!(prev, None);
let prev = x.take_if(|v| *v == 43);
assert_eq!(x, None);
assert_eq!(prev, Some(43));1.31.0 · Sourcepub fn replace(&mut self, value: T) -> Option<T>
pub fn replace(&mut self, value: T) -> Option<T>
Replaces the actual value in the option by the value given in parameter,
returning the old value if present,
leaving a Some in its place without deinitializing either one.
Examples
let mut x = Some(2);
let old = x.replace(5);
assert_eq!(x, Some(5));
assert_eq!(old, Some(2));
let mut x = None;
let old = x.replace(3);
assert_eq!(x, Some(3));
assert_eq!(old, None);Trait Implementations
Sourceimpl<T> Deref for Range<T>where
T: RangeSubType,
impl<T> Deref for Range<T>where
T: RangeSubType,
Sourceimpl<T> DerefMut for Range<T>where
T: RangeSubType,
impl<T> DerefMut for Range<T>where
T: RangeSubType,
Sourceimpl<T> From<RangeInclusive<T>> for Range<T>where
T: RangeSubType,
impl<T> From<RangeInclusive<T>> for Range<T>where
T: RangeSubType,
Sourcefn from(value: RangeInclusive<T>) -> Range<T>
fn from(value: RangeInclusive<T>) -> Range<T>
Sourceimpl<T> From<RangeToInclusive<T>> for Range<T>where
T: RangeSubType,
impl<T> From<RangeToInclusive<T>> for Range<T>where
T: RangeSubType,
Sourcefn from(value: RangeToInclusive<T>) -> Range<T>
fn from(value: RangeToInclusive<T>) -> Range<T>
Sourceimpl<T> FromDatum for Range<T>where
T: RangeSubType,
impl<T> FromDatum for Range<T>where
T: RangeSubType,
Sourceconst GET_TYPOID: bool = false
const GET_TYPOID: bool = false
from_datum?Sourceunsafe fn from_datum(datum: Datum, is_null: bool) -> Option<Self>where
Self: Sized,
unsafe fn from_datum(datum: Datum, is_null: bool) -> Option<Self>where
Self: Sized,
Sourceunsafe fn from_datum_in_memory_context(
memory_context: PgMemoryContexts,
datum: Datum,
is_null: bool,
typoid: Oid,
) -> Option<Self>where
Self: Sized,
unsafe fn from_datum_in_memory_context(
memory_context: PgMemoryContexts,
datum: Datum,
is_null: bool,
typoid: Oid,
) -> Option<Self>where
Self: Sized,
FromDatum::from_datum(...) from within that context. Read moreSourceunsafe fn try_from_datum(
datum: Datum,
is_null: bool,
type_oid: Oid,
) -> Result<Option<Self>, TryFromDatumError>
unsafe fn try_from_datum( datum: Datum, is_null: bool, type_oid: Oid, ) -> Result<Option<Self>, TryFromDatumError>
try_from_datum is a convenience wrapper around FromDatum::from_datum that returns a
a Result around an Option, as a Datum can be null. It’s intended to be used in
situations where the caller needs to know whether the type conversion succeeded or failed. Read moreSourceunsafe fn try_from_datum_in_memory_context(
memory_context: PgMemoryContexts,
datum: Datum,
is_null: bool,
type_oid: Oid,
) -> Result<Option<Self>, TryFromDatumError>
unsafe fn try_from_datum_in_memory_context( memory_context: PgMemoryContexts, datum: Datum, is_null: bool, type_oid: Oid, ) -> Result<Option<Self>, TryFromDatumError>
try_from_datum that switches to the given context to convert from DatumSourceimpl<T> IntoDatum for Range<T>where
T: RangeSubType,
impl<T> IntoDatum for Range<T>where
T: RangeSubType,
Sourceimpl SqlTranslatable for Range<AnyNumeric>
impl SqlTranslatable for Range<AnyNumeric>
fn argument_sql() -> Result<SqlMapping, ArgumentError>
fn return_sql() -> Result<Returns, ReturnsError>
fn type_name() -> &'static str
fn variadic() -> bool
fn optional() -> bool
fn entity() -> FunctionMetadataTypeEntity
Sourceimpl SqlTranslatable for Range<Date>
impl SqlTranslatable for Range<Date>
fn argument_sql() -> Result<SqlMapping, ArgumentError>
fn return_sql() -> Result<Returns, ReturnsError>
fn type_name() -> &'static str
fn variadic() -> bool
fn optional() -> bool
fn entity() -> FunctionMetadataTypeEntity
Sourceimpl<const P: u32, const S: u32> SqlTranslatable for Range<Numeric<P, S>>
impl<const P: u32, const S: u32> SqlTranslatable for Range<Numeric<P, S>>
fn argument_sql() -> Result<SqlMapping, ArgumentError>
fn return_sql() -> Result<Returns, ReturnsError>
fn type_name() -> &'static str
fn variadic() -> bool
fn optional() -> bool
fn entity() -> FunctionMetadataTypeEntity
Sourceimpl SqlTranslatable for Range<Timestamp>
impl SqlTranslatable for Range<Timestamp>
fn argument_sql() -> Result<SqlMapping, ArgumentError>
fn return_sql() -> Result<Returns, ReturnsError>
fn type_name() -> &'static str
fn variadic() -> bool
fn optional() -> bool
fn entity() -> FunctionMetadataTypeEntity
Sourceimpl SqlTranslatable for Range<TimestampWithTimeZone>
impl SqlTranslatable for Range<TimestampWithTimeZone>
fn argument_sql() -> Result<SqlMapping, ArgumentError>
fn return_sql() -> Result<Returns, ReturnsError>
fn type_name() -> &'static str
fn variadic() -> bool
fn optional() -> bool
fn entity() -> FunctionMetadataTypeEntity
Sourceimpl SqlTranslatable for Range<i32>
impl SqlTranslatable for Range<i32>
fn argument_sql() -> Result<SqlMapping, ArgumentError>
fn return_sql() -> Result<Returns, ReturnsError>
fn type_name() -> &'static str
fn variadic() -> bool
fn optional() -> bool
fn entity() -> FunctionMetadataTypeEntity
Sourceimpl SqlTranslatable for Range<i64>
impl SqlTranslatable for Range<i64>
fn argument_sql() -> Result<SqlMapping, ArgumentError>
fn return_sql() -> Result<Returns, ReturnsError>
fn type_name() -> &'static str
fn variadic() -> bool
fn optional() -> bool
fn entity() -> FunctionMetadataTypeEntity
impl<T> Eq for Range<T>where
T: Eq + RangeSubType,
impl<T> StructuralPartialEq for Range<T>where
T: RangeSubType,