#[repr(i32)]pub enum SpiErrorCodes {
Show 13 variants
Connect = -1,
Copy = -2,
OpUnknown = -3,
Unconnected = -4,
Cursor = -5,
Argument = -6,
Param = -7,
Transaction = -8,
NoAttribute = -9,
NoOutFunc = -10,
TypUnknown = -11,
RelDuplicate = -12,
RelNotFound = -13,
}Expand description
These match the Postgres #defined constants prefixed SPI_ERROR_* that you can find in pg_sys.
It is hypothetically possible for a Postgres-defined status code to be 0, AKA NULL, however,
this should not usually occur in Rust code paths. If it does happen, please report such bugs to the pgx repo.
Variants
Connect = -1
Copy = -2
OpUnknown = -3
Unconnected = -4
Cursor = -5
Argument = -6
Param = -7
Transaction = -8
NoAttribute = -9
NoOutFunc = -10
TypUnknown = -11
RelDuplicate = -12
RelNotFound = -13