pub struct SpiHeapTupleData { /* private fields */ }Expand description
Represents the set of pg_sys::Datums in a pg_sys::HeapTuple
Implementations
Sourceimpl SpiHeapTupleData
impl SpiHeapTupleData
Sourcepub unsafe fn new(
tupdesc: *mut TupleDescData,
htup: *mut HeapTupleData,
) -> Result<Option<SpiHeapTupleData>, Error>
pub unsafe fn new( tupdesc: *mut TupleDescData, htup: *mut HeapTupleData, ) -> Result<Option<SpiHeapTupleData>, Error>
Create a new SpiHeapTupleData from its constituent parts
Safety
This is unsafe as it cannot ensure that the provided tupdesc and htup arguments
are valid, palloc’d pointers.
Sourcepub fn get<T>(&self, ordinal: usize) -> Result<Option<T>, Error>
pub fn get<T>(&self, ordinal: usize) -> Result<Option<T>, Error>
Get a typed value from this HeapTuple by its ordinal position.
The ordinal position is 1-based
Errors
Returns a Error::DatumError if the desired Rust type is incompatible
with the underlying Datum
Sourcepub fn get_by_name<T, S>(&self, name: S) -> Result<Option<T>, Error>
pub fn get_by_name<T, S>(&self, name: S) -> Result<Option<T>, Error>
Get a typed value from this HeapTuple by its name in the resultset.
Errors
Returns a Error::DatumError if the desired Rust type is incompatible
with the underlying Datum
Sourcepub fn get_datum_by_ordinal(
&self,
ordinal: usize,
) -> Result<&SpiHeapTupleDataEntry, Error>
pub fn get_datum_by_ordinal( &self, ordinal: usize, ) -> Result<&SpiHeapTupleDataEntry, Error>
Get a raw Datum from this HeapTuple by its ordinal position.
The ordinal position is 1-based.
Errors
If the specified ordinal is out of bounds a [Error::SpiError(SpiError::NoAttribute)] is returned
Sourcepub fn get_datum_by_name<S>(
&self,
name: S,
) -> Result<&SpiHeapTupleDataEntry, Error>
pub fn get_datum_by_name<S>( &self, name: S, ) -> Result<&SpiHeapTupleDataEntry, Error>
Sourcepub fn set_by_ordinal<T>(
&mut self,
ordinal: usize,
datum: T,
) -> Result<(), Error>where
T: IntoDatum,
pub fn set_by_ordinal<T>(
&mut self,
ordinal: usize,
datum: T,
) -> Result<(), Error>where
T: IntoDatum,
Set a datum value for the specified ordinal position
Errors
If the specified ordinal is out of bounds a SpiErrorCodes::NoAttribute is returned
Sourcepub fn set_by_name<T>(&mut self, name: &str, datum: T) -> Result<(), Error>where
T: IntoDatum,
pub fn set_by_name<T>(&mut self, name: &str, datum: T) -> Result<(), Error>where
T: IntoDatum,
pub fn columns(&self) -> usize